<?php
// $Id: ec_cart.theme.inc,v 1.1 2008/10/12 07:03:51 gordon Exp $

/**
 * @file
 * eC Cart themes
 */

/**
 * Tall user that their cart is empty
 *
 * @return
 *  string, HTML telling the cart is empty
 * @ingroup themable
 */
function theme_ec_cart_empty() {
  return '<p>'. t('Your shopping cart is empty. You may !start_shopping.',
    array('!start_shopping' => l(t('start shopping'),
    variable_get('ec_goto_cart_empty', '')))) .'</p>';
}

/**
 * Returns a themed shopping cart form.
 * @ingroup themeable
 */
function theme_ec_cart_view_form($form) {
  $total = 0;
  $header = array(
    array('class' => 'desc', 'data' => t('Items')),
    array('class' => 'price', 'data' => t('Price')),
    array('class' => 'qty', 'data' => t('Qty')),
  );

  $extra = array_filter($form['products'], '_ec_cart_form_filter_extra');
  if (!empty($extra)) {
    $header[] =  array('class' => 'extra', 'data' => '<!--extra-->');
  }
  $header[] = array('class' => 'total', 'data' => t('Total'));
  $header[] = array('class' => 'ops', 'data' => '<!--ops-->');
  $cart_total = 0;

  foreach (element_children($form['products']) as $nid) {
    $item = $form['products'][$nid]['#node'];

    // Prepare some description.
    $desc = '<p>'. drupal_render($item->title) .'</p>';
    // TODO: this is too hard coded.
    if (!empty($form['products'][$nid]['recurring'])) {
      $desc .= '<p class="item-recurring">'. drupal_render($form['products'][$nid]['recurring']) .'</p>';
    }
    if (!empty($form['products'][$nid]['availability'])) {
      $desc .= '<p class="item-availability">'. drupal_render($form['products'][$nid]['availability']) .'</p>';
    }
    // Desc, price, qty
    $row = array(
      array('class' => 'desc', 'data' => $desc),
      array('class' => 'price', 'data' => format_currency($item->price)),
      array('class' => 'qty', 'data' => $item->qty ? drupal_render($form['products'][$nid]['qty']) : ''),
    );
    // Extra stuff
    if (!empty($extra) && !empty($item->data)) {
      $row[] = array('class' => 'extra', 'data' => drupal_render($form['products'][$nid]['qty']));
    }
    elseif (!empty($extra)) {
      $row[] = '';
    }
    // Total and remove link
    $row[] = array('class' => 'total', 'data' => format_currency($item->total));
    $row[] = array('class' => 'ops', 'data' => l(t('Remove'), "cart/delete/$nid", array('query' => drupal_get_destination())));
    $rows[] = array('class' => 'cart-item', 'id' => 'item-'. $nid, 'data' => $row);
    $cart_total += $item->total;
  }
  // Final row.
  $rows[] = array('id' => 'cart-final', 'data' => array(
    array('data' => '', 'colspan' => $extra ? 4 : 3),
    array('id' => 'cart-total', 'data' => format_currency($cart_total)),
    '',
  ));
  $output = theme('table', $header, $rows);
  $output = '<div id="cart-view-page">'. $output . drupal_render($form) .'</div>';
  return $output;
}

/**
 * Themes the cart block. Called by ec_cart_display_block().
 * @ingroup themeable
 */
function theme_ec_cart_display_block($cart, $items = array()) {
  drupal_add_css(drupal_get_path('module', 'cart') .'/cart.css');
  $output = '';

  // If $cart is FALSE, then we are just setting a link to the cart.
  if (!$cart) {
    return '<div class="cart-link">'. l(t('View your cart'), 'cart/view') .'</div>';
  }

  // Shopping cart summary.
  $output .= '<div class="item-count">'. t('%items in !your_cart',
    array('%items' => format_plural(count($items), '1 item', '@count items'),
    '!your_cart' => l(t('Your cart'), 'cart/view'))) .'</div>';

  // Shopping cart list of items.
  $output .= '<div class="cart-items">';
  if (count($items)) {
    foreach ($items AS $item) {
      $output .= '<div id="cart-item-'. $item->nid .'" class="cart-item">';
      $output .= ec_cart_display_block_item($item);
      $output .= '</div>';
    }
    $output .= '</div><div class="total">'. format_currency($cart['total']) ."</div>\n";
    $output .= '<div class="checkout">'. t('Ready to <a href="!checkout-url">checkout</a>?', array('!checkout-url' => url('checkout/cart'))) .'</div>';
  }
  else {
    $output .= '<!-- empty cart --></div>';
  }

  // A class wrapper for items vs no items.
  //$output = '<div class="cart-'. (count($items)) ? 'full' : 'empty' .'">'. $output .'</div>';
  return $output;
}

/**
 * Themes the cart block item
 * @ingroup themeable
 */
function theme_ec_cart_display_block_item($item) {
  $output = l($item->title . (($item->theme_qty > 1) ? ' x '. $item->theme_qty : ''), $item->theme_path);
  $output .= '<span class="cart-line-total">'. format_currency($item->theme_price) .'</span>';
  return $output;
}

/**
 * Themes the cart block title
 * @ingroup themeable
 */
function theme_ec_cart_display_block_title($item_count, $text = 'Shopping Cart') {
  return l($text, 'cart/view', array('title' => t('View your cart items.')), NULL, NULL, FALSE, TRUE);
}

