<?php
// $Id: ec_store.admin.inc,v 1.3.2.1 2008/11/04 12:39:42 gordon Exp $

/**
 * @file
 * store.admin.inc
 * All store admin functions.
 */

/**
 * Main store settings page, at 'admin/ecsettings/store'
 */
function ec_store_ec_settings() {
  $site_mail = variable_get('site_mail', ini_get('sendmail_from'));
  if (empty($site_mail)) {
    $site_mail = 'no e-mail address found';
  }

  $form['ec_order_overview'] = array(
    '#type'           => 'textarea',
    '#title'          => t('Order history overview'),
    '#default_value'  => variable_get('ec_order_overview', ''),
    '#cols'           => 70,
    '#rows'           => 5,
    '#description'    => t('This text will be displayed at the top of users\' order history pages.'),
  );

  $form['product_cart_addition_by_link'] = array(
    '#type'           => 'checkbox',
    '#title'          => t('Users adds product to cart using link.'),
    '#default_value'  => variable_get('product_cart_addition_by_link', 1),
    '#description'    => t('If this field is unchecked a form will be placed at the bottom of the product view which will allow the user select the quantity and other information.'),
  );

  $form['ec_display_email'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display email address for non-anonymous customers'),
    '#default_value' => variable_get('ec_display_email', 1),
  );

  // Links settings
  $form['ec_links'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Links'),
    '#collapsible'    => TRUE,
    '#collapsed'      => TRUE
  );
  $form['ec_links']['ec_goto_cart_change'] = array(
    '#type'           => 'textfield',
    '#title'          => t('After add an item link'),
    '#default_value'  => variable_get('ec_goto_cart_change', 'cart/view'),
    '#description'    => t('Enter the user destination after adding a product to the cart (eg. "node/1"). If left blank, the user will remain on the same page and a message will alert them that the item has been added to their cart. To send the user to cart, enter "cart/view".'),
  );
  $form['ec_links']['ec_goto_cart_empty'] = array(
    '#type'           => 'textfield',
    '#title'          => t('Empty cart link'),
    '#default_value'  => variable_get('ec_goto_cart_empty', ''),
    '#description'    => t('If the user cart is empty, you can send him/her to a given page.'),
  );

  // Transaction Notices settings
  $form['ec_notices'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Transaction notices'),
    '#collapsible'    => TRUE,
    '#collapsed'      => TRUE,
    '#description'    => t('These settings control whether the site administrator will receive email notifications of transaction reports, and the frequency thereof.'),
  );
  $form['ec_notices']['payment_notices'] = array(
    '#type'           => 'radios',
    '#title'          => t('Transaction e-mail notices'),
    '#default_value'  => variable_get('payment_notices', 0),
    '#options'        => array(t('Disabled'), t('Enabled')),
    '#description'    => t('Enable or disable transaction notifications. If enabled, the site administrator (%site_mail) will receive an email summarizing the status of all orders. The frequency of emails is dependent on how often the site administrator has scheduled to run cron jobs.  Finally, The site administrator e-mail address can be changed in the <a href="!general_config_screen">general configuration screen</a>,',
      array('%site_mail' => $site_mail,
      '!general_config_screen' => url('admin/settings')))
  );
  $form['ec_notices']['ecommerce_store_summary_email_frequency'] = array(
    '#type'           => 'select',
    '#title'          => t('Frequency of notices'),
    '#options'        => drupal_map_assoc(
      array(0, 60*60, 60*60*12, 60*60*24, 60*60*24*5, 60*60*24*7),
      'format_interval'),
    '#default_value'  => variable_get('ecommerce_store_summary_email_frequency', 60*60*24),
    '#description'    => t('As long as cron is running, this ensures a transaction report is sent no more than once in this period. Thus, even if cron runs every hour, you will still only receive once transaction notification e-mail.'),
  );

  return system_settings_form($form);
}


// -------------------------------------------------------------------------
// TODO: move these functions. They are store management, not ecommerce settings.

/**
 * Called by store_transactions_advanced_search_form()
 */
function _ec_store_transactions_advanced_search_form($query) {
  $form['keywords1'] = array(
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  $form['keywords1']['txnid'] = array(
    '#type' => 'textfield',
    '#title' => t('Transaction ID'),
    '#default_value' => empty($query['txnid']) ? '' : $query['txnid'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('The ID number of a transaction.')
  );
  $form['keywords1']['nid'] = array(
    '#type' => 'textfield',
    '#title' => t('Product ID'),
    '#default_value' => empty($query['nid']) ? '' : $query['nid'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Find a transaction that has the following product(s).')
  );
  $form['keywords2'] = array('#prefix' => '<div class="criterion">', '#suffix' => '</div>');
  $form['keywords2']['nid_expires'] = array(
    '#type' => 'textfield',
    '#title' => t('Product expiration'),
    '#default_value' => empty($query['nid_expires']) ? '' : $query['nid_expires'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date product expires.')
  );
  $form['keywords2']['date'] = array(
    '#type' => 'textfield',
    '#title' => t('Date created'),
    '#default_value' => empty($query['date']) ? '' : $query['date'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date transaction was created.')
  );
  $form['keywords2']['duedate'] = array(
    '#type' => 'textfield',
    '#title' => t('Date due'),
    '#default_value' => empty($query['duedate']) ? '' : $query['duedate'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date transaction was due.')
  );
  $form['keywords2']['gross'] = array(
    '#type' => 'textfield',
    '#title' => t('Total cost'),
    '#default_value' => empty($query['gross']) ? '' : $query['gross'],
    '#size' => 30,
    '#maxlength' => 255
  );
  $form['allocation'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Allocation status'),
    '#default_value' => empty($query['allocation']) ? '' : $query['allocation'],
    '#options' => ec_store_transaction_allocation(),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  $form['workflow'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Workflow'),
    '#default_value' => empty($query['workflow']) ? '' : $query['workflow'],
    '#options' => ec_store_transaction_workflow(),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  $form['ptype'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Product type'),
    '#default_value' => empty($query['ptype']) ? '' : $query['ptype'],
    '#options' => product_ptypes_get('names'),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );

  return $form;
}

/**
 * Overview Widget that shows some orders data.
 *
 * @param $widget
 *   Array (optional), The original widget options
 * @return
 *   Array, the widget data
 */
function _ec_store_widget_orders($widget = array()) {
  $num_pending    = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE (allocation = %d OR workflow = %d) AND workflow IN (%d, %d, %d, %d)',
    EC_ALLOC_PENDING, EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED,
    EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE));
  $num_part_paid      = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} WHERE allocation = %d', EC_ALLOC_PART));
  $num_processing = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE allocation = %d AND workflow IN (%d, %d, %d, %d)', EC_ALLOC_COMPLETED,
    EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE));
  $num_completed  = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE allocation = %d and workflow = %d', EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
  $num_alloc_cancel = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} WHERE allocation IN (%d, %d) AND workflow = %d', EC_ALLOC_PART, EC_ALLOC_COMPLETED, EC_WORKFLOW_CANCELED));

  // Build the content table
  $content = theme('table', NULL, array(
    array(
      array('data' => l(t('Payments pending'), 'admin/store/transaction',
        array(), 'allocation=0,1&workflow=1,2,3,4'), 'align' => 'right',
        'title' => t('Payment status: Pending, workflow: Not Complete')),
      array('data' => $num_pending, 'align' => 'right')
    ),
    array(
      array('data' => l(t('Part Payments'), 'admin/store/transaction',
        array(), 'allocation=1'), 'align' => 'right',
        'title' => t('Payment status: Part')),
      array('data' => $num_part_paid, 'align' => 'right')
    ),
    array(
      array('data' => l(t('Orders needing action'), 'admin/store/transaction',
        array(), 'allocation=2&workflow=1,2,3,4'), 'align' => 'right',
        'title' => t('Payment status: Complete, workflow: Not Complete')),
      array('data' => $num_processing, 'align' => 'right')
    ),
    array(
      array('data' => l(t('Completed'), 'admin/store/transaction',
        array(), 'workflow=6'), 'align' => 'right'),
      array('data' => $num_completed, 'align' => 'right'),
    ),
    array(
      array('data' => l(t('Canceled with Allocated funds'), 'admin/store/transaction',
        array(), 'workflow=5&allocation=1,2'), 'align' => 'right'),
      array('data' => $num_alloc_cancel, 'align' => 'right'),
    )
  ));

  return $content;
}

/**
 * Overview Widget that shows some sales data.
 *
 * @param $widget
 *   Array (optional), The original widget options
 * @return
 *   Array, the widget data
 */
function _ec_store_widget_sales($widget = array()) {
  $beg_month   = mktime(0, 0, 0, date('m'), 1, date('Y'));
  $beg_year    = mktime(0, 0, 0, 1, 1, date('Y'));
  $num_orders  = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE created >= %d AND allocation = %d AND workflow = %d', $beg_month,
    EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
  $mon_sales   = format_currency(db_result(db_query('SELECT SUM(gross)
    FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d',
    $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)));
  $year_sales  = format_currency(db_result(db_query('SELECT SUM(gross)
    FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d',
    $beg_year, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)));

  // Build the content table
  $content = theme('table', NULL, array(
    array(
      array('data' => l(t('Monthly orders'), 'admin/store/transaction',
        array(), 'date=>'. date('m/d/Y', $beg_month)), 'align' => 'right'),
      array('data' => $num_orders, 'align' => 'right')
    ),
    array(
      array('data' => t('Monthly sales'), 'align' => 'right'),
      array('data' => $mon_sales, 'align' => 'right'),
    ),
    array(
      array('data' => t('Yearly sales'), 'align' => 'right'),
      array('data' => $year_sales, 'align' => 'right'),
    ),
  ));

  return $content;
}

/**
 * Called by theme_store_invoice()
 */
function _theme_ec_store_invoice($txn, $print_mode = TRUE, $trial = FALSE) {
  global $base_url;

  $header = array();
  $row    = array();

  if (empty($txn->mail) && $txn->uid > 0) {
    $txn->mail = db_result(db_query('SELECT mail FROM {users} WHERE uid = %d', $txn->uid));
  }

  if (!empty($txn->items)) {
    $header = array(t('Quantity'), t('Item'), t('Price'));

    $shippable = FALSE;
    foreach ($txn->items as $p) {
      $prod = ec_product_load($p);
      if (product_is_shippable($p->vid)) {
        $shippable = TRUE;
      }
      $price = ec_store_adjust_misc($txn, $p);

      $subtotal += (product_has_quantity($p) ? $p->qty * $price : $price);
      $details = '';
      if (0 && is_array($p->data)) {
        foreach ($p->data as $key => $value) {
          if (!empty($value)) {
            $items[] = '<strong>'. check_plain($key) .': </strong>'. check_plain($value);
          }
        }
        if (!empty($items)) {
          $details = theme('item_list', $items);
        }
      }

      $row[] = array(array('data' => $p->qty, 'align' => 'center', 'valign' => 'top'),
        '<em>'. check_plain($p->title) .'</em> '. (($prod->sku != '') ?
        '['. check_plain($prod->sku) .']' : '') .'<br />'. $details,
        array('data' => format_currency($price), 'valign' => 'top', 'align' => 'right'));
    }

    if (is_array($txn->misc)) {
      foreach ($txn->misc as $misc) {
        if (empty($misc->seen)) {
          $price = $misc->qty ? $misc->price * $misc->qty : $misc->price;
          $row[] = array(array('data' => "<strong>{$misc->description}</strong>: ".
            format_currency($price), 'colspan' => 3, 'align' => 'right'));
        }
      }
    }

    $row[] = array(array('data' => '<hr size="1" noshade="noshade" />', 'colspan' => 3, 'align' => 'right'));
    $row[] = array(array('data' => '<strong>'. t('Total:') .'</strong> '.
      format_currency(ec_store_transaction_calc_gross($txn)),
      'colspan' => 3, 'align' => 'right'));
  }

  $payment_info  = '<div><strong>'. t('Ordered On:') .'</strong> '.
    format_date($txn->created) .'</div>';
  if (!empty($txn->duedate)) {
    $payment_info .= '<div><strong>'. t('Due Date:') .'</strong> '.
      format_date($txn->duedate) .'</div>';
  }
  $payment_info .= '<div><strong>'. t('Transaction ID:') .'</strong> '.
    ($trial ? t('Trial Invoice - Not Yet Posted') : $txn->txnid) .'</div>';

  $css        = base_path() . drupal_get_path('module', 'store') .'/invoice.css';
  $site_name  = t('%site-name Invoice', array('%site-name' => variable_get('site_name', 'drupal')));

  module_load_include('inc', 'ec_store', 'ec_store');
  if ($shipping_to = theme('ec_store_format_address', $txn, 'shipping', 'html')) {
    $shipping_label = t('Shipping to');
  }
  if ($billing_to = theme('ec_store_format_address', $txn, 'billing', 'html')) {
    $billing_label = t('Billing to');
  }

  if (!empty($txn->ship)) {
    $shipping_method_label = t('Shipping method:');
    $shipping_method = ec_store_format_shipping_method($txn);
  }
  $email_label = t('E-mail:');
  $items_label = t('Items ordered');
  $items_view = theme('table', $header, $row, array('cellpadding' => 3, 'cellspacing' => 3));

  $payment_label = t('Payment Info');

  if (!empty($print_mode)) {
    $output .= <<<EOD
<html>
  <head>
    <style type="text/css" media="all">@import url('$css');</style>
  </head>
  <body>
EOD;
  }

$output .= <<<EOD
    <h1>$site_name</h1>

    <table cellspacing="5">
      <tr>
        <th align="left">$shipping_label</th>
        <th align="left">$billing_label</th>
      </tr>
      <tr>
        <td>$shipping_to</td>
        <td>$billing_to</td>
      </tr>
    </table>

    <p><strong>$shipping_method_label</strong> $shipping_method</p>
    <p><strong>$email_label</strong> $txn->mail</p>

    <h2>$items_label</h2>
    $items_view

    <h2>$payment_label</h2>
    $payment_info
EOD;

if (!empty($print_mode)) {
  $output .= <<<EOD
    </body>
  </html>
EOD;
}

  if (empty($print_mode)) {
    return $output;
  }
  echo $output;
}


