<?php
// $Id: ec_store.inc,v 1.5.2.1 2008/11/20 02:30:43 gordon Exp $

/**
 * @file
 * store.inc
 * Code here is not specific to any process, might be any user, and is not run infrequently.
 */

// -------------------------------------------------------------------------
// Infrequent processing

/**
 * Implementation of hook_cron().
 */
function _ec_store_cron() {
  // Any item whose workflow is 'shipped', move it to 'completed'. This does
  // seem presumptous. Ideally we'd email the user a list of shipped items for
  // day and then adjust the shipping workflow after a given time period, such
  // as 2 week (in case of returns or something) rather than automatically
  // closing it here.
  db_query('UPDATE {ec_transaction} SET workflow = %d
    WHERE workflow = %d AND allocation = %d', EC_WORKFLOW_COMPLETED,
    EC_WORKFLOW_SHIPPED, EC_ALLOC_COMPLETED);

  if (variable_get('payment_notices', 1) and
      (time() >= (variable_get('ecommerce_store_summary_email_sent', 0) +
      variable_get('ecommerce_store_summary_email_frequency', 60 * 60 * 24)))) {
    global $base_url;

    // Find order(s) with a 'received' payment status needs action.
    if ($count = db_result(db_query('SELECT COUNT(st.txnid)
      FROM {ec_transaction} AS st WHERE st.allocation = %d AND
      (st.workflow = %d OR st.workflow = %d OR st.workflow = %d OR st.workflow = %d)',
      EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED,
      EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE))) {

      $beg_month    = mktime(0, 0, 0, date('m'), 1, date('Y'));
      $mon_orders  = db_result(db_query('SELECT COUNT(txnid)
        FROM {ec_transaction} where created >= %d AND allocation = %d AND
        workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
      $mon_sales    = '$'. number_format(db_result(db_query('SELECT SUM(gross)
        FROM {ec_transaction} where created >= %d AND allocation = %d AND
        workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)), 2);

      $buf = t('@count order(s) with a "cleared" payment status needs action:',
        array('@count' => $count)) ."\n";
      $result = db_query('SELECT * FROM {ec_transaction} AS st
        WHERE st.allocation = %d AND st.workflow IN (%d, %d, %d, %d)
        ORDER BY st.txnid DESC', EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED,
        EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE);
      while ($data = db_fetch_object($result)) {

        $buf .= t('Order #@txnid created @timestamp ago', array('@txnid' => $data->txnid,
          '@timestamp' => format_interval(time() - $data->created, 2))) ."\n".
          t('Workflow status: @transaction-workflow',
          array('@transaction-workflow' => transaction_get_workflow($data->workflow))) ."\n".
          t('Email: @email', array('@email' => $data->mail)) ."\n".
          t('Total: @gross', array('@gross' => format_currency($data->gross))) ."\n";

        $r = db_query('SELECT stp.*, st.allocation, st.workflow
          FROM {ec_transaction} AS st, {ec_transaction_product} AS stp
          WHERE st.txnid = %d AND st.txnid = stp.txnid
          ORDER BY st.txnid DESC', $data->txnid);
        $items = NULL;
        $has_shippable_item = FALSE;
        while ($item = db_fetch_object($r)) {
          if ($product = ec_product_load($item)) {
            foreach ($product as $key => $value) {
              if ($key != 'title' && $key != 'price') {
                $item->$key = $value;
              }
            }
          }
          if (product_is_shippable($item->vid)) {
            $has_shippable_item = TRUE;
          }

          $sku = ($item->sku) ? " [$item->sku]" : "";
          $items .= t('@order of @title at @price each', array(
            '@order' => format_plural($item->qty, '1 order', '@count orders'),
            '@title' => $item->title . $sku,
            '@price' => format_currency($item->price)));
        }

        // If the transaction has no shippable items and the payment status is
        // 'complete' and the workflow is 'transaction received', set the
        // workflow to 'complete'
        if (empty($has_shippable_item) && $data->allocation == EC_ALLOC_COMPLETED && $data->workflow == EC_WORKFLOW_RECEIVED) {
          db_query('UPDATE {ec_transaction} SET workflow = %d WHERE txnid = %d', EC_WORKFLOW_COMPLETED, $data->txnid);
        }

        $shipping_to = theme('ec_store_format_address', $data, 'shipping', 'text');
        $billing_to = theme('ec_store_format_address', $data, 'billing', 'text');

        $buf .= t("Shipping to:\n!shipping_address", array('!shipping_address' => $shipping_to)) ."\n".
          t("Billing to:\n!billing_address", array('!billing_address' => $billing_to)) ."\n".
          "$items\n".
          t('Update:') ."$base_url/admin/store/transaction/edit/$data->txnid\n".
          "=====\n";

      }

      $output = $buf;
      $output .= t('(These orders will no longer be listed once the transaction workflow is set to either "Completed" or "Canceled")') .
        "\n". t("Monthly Summary\nNumber of completed orders: @monthly-orders\nSales: @monthly-sales",
        array('@monthly-orders' => $mon_orders, '@monthly-sales' => $mon_sales)) ."\n".
        t('Want more reports? Visit @base_url/admin/store', array('@base_url' => $base_url)) ."\n";

      $subject    = t('@site-name transaction summary', array('@site-name' => variable_get('site_name', 'Drupal')));
      $to         = variable_get('site_mail', ini_get('sendmail_from'));
      $from       = variable_get('site_mail', ini_get('sendmail_from'));
      $headers    = array(
        'From'        => $from,
        'Reply-to'    => $from,
        'X-Mailer'    => 'PHP',
        'Return-path' => $from,
        'Errors-to'   => $from,
      );

      // TODO: drupal_mail('store', 'ec_order_summary', $to, $subject, $output, $from, $headers);
      variable_set('ecommerce_store_summary_email_sent', time());
    }
  } // End receive payment notices.
}

// -------------------------------------------------------------------------
// Unclassified


function ec_store_transaction_overview_save($edit) {

  if (!is_numeric($edit['uid'])) {
    $edit['uid'] = db_result(db_query("SELECT uid FROM {users} WHERE name = '%s'", $edit['uid']));
  }
  return db_query('UPDATE {ec_transaction} SET uid = %d, workflow = %d,
    payment_status = %d, created = %d, expires = %d WHERE txnid = %d', $edit['uid'],
    $edit['workflow'], $edit['payment_status'], $edit['created'],
    ($edit['expires'] != 0) ? $edit['expires'] : 0, $edit['txnid']);
}

/**
 * Saves all addresses in transaction using ec_store_transaction_address_save
 *
 * @param $txn
 *   Object, transaction with address array
 */
function ec_store_transaction_addresses_save($txn) {
  $txn = (object)$txn; // just to be sure

  // If we where not passed a valid transaction and we can't save.
  if (!isset($txn->txnid) || !is_array($txn->address)) {
    return FALSE;
  }
  foreach ($txn->address as $type => $address) {
    if (is_array($address)) {
      $address = (object)$address;
    }
    $address->txnid = $txn->txnid;
    ec_store_transaction_address_save($address, $type);
  }
}

/**
 * Saves an individual address to the passed transaction
 *
 * @param $address
 *   Object, address object
 * @param $type
 *   String, address type(corresponds to type field in database)
 */
function ec_store_transaction_address_save($address, $type) {
  $address = (object)$address; // just to be sure
  $address->type = $type;

  drupal_write_record('ec_transaction_address', $address, array('txnid', 'type'));
  if (!db_affected_rows()) {
    drupal_write_record('ec_transaction_address', $address);
  }
}

function ec_store_transaction_item_save($edit) {

  foreach ($edit['item'] as $key => $value) {
    db_query("UPDATE {ec_transaction_product} SET qty = %d, price = %f, title = '%s' WHERE txnid = '%d AND nid = %d", $value['qty'], $value['price'], $value['title'],
      $edit['txnid'], $key);
  }
}

function ec_store_transaction_validate_overview(&$edit) {

  global $user;

  $errors = array();

  if (!is_numeric($edit['uid']) && db_result(db_query("SELECT COUNT(uid) FROM {users} WHERE name = '%s'", $edit['uid'])) == 1) {
    $edit['uid'] = db_result(db_query("SELECT uid FROM {users} WHERE name = '%s'", $edit['uid']));
  }

  /* Make sure we have a valid username or id */
  if ($edit['uid'] == variable_get("anonymous", "Anonymous")) {
    $edit['uid'] = 0;
  }
  elseif (!is_numeric($edit['uid']) && db_result(db_query("SELECT COUNT(uid) FROM {users} WHERE name = '%s'", $edit['uid'])) == 0) {
    $errors['uid'] = t('Invalid username');
  }

  if (empty($edit['mail']) && $edit['uid'] != 0) {
    $edit['mail'] = $user->mail;
  }

  if ($edit['mail'] && ($msg = user_validate_mail($edit['mail']))) {
    $errors['mail'] = $msg;
  }

  $edit['changed'] = $edit['changed'] ? strtotime((int) $edit['changed']) : time();
  if ($edit['changed'] === FALSE || $edit['changed'] == -1) {
    $errors['changed'] = t('Invalid changed date. Please use the form "18 January 2005"');
  }

  foreach ($errors as $name => $message) {
    form_set_error($name, $message);
  }
  return count($errors) == 0;
}

/**
 * Validate addressess on certain admin forms I think...
 * @todo This hasn't been converted to the new address system. Fix this
 */
function ec_store_transaction_validate_addresses(&$edit, $validate = 'all') {

  global $user;

  $errors = array();

  if ($user->uid == 0 && variable_get('ec_anon_policy', ECANON_POLICY_DEFAULT) != ECANON_POLICY_NEVER) {
    if ($edit['shippable'] && !($edit['shipping_firstname'] && $edit['shipping_lastname'] && $edit['shipping_street1'] && $edit['shipping_zip'] && $edit['shipping_city'] && $edit['shipping_state'])) {
      $shipping_error = t('Please fill out all shipping information fields.');
    }

    if (!($edit['billing_firstname'] && $edit['billing_lastname'] && $edit['billing_street1'] && $edit['billing_zip'] && $edit['billing_city'] && $edit['billing_state'])) {
      $billing_error = t('Please fill out all billing information fields.');
    }

    if (empty($shipping_error) && !empty($billing_error)) {
      $reset_bc = FALSE;
      if ($validate == 'all') {
        foreach ($edit as $key => $value) {
          if (drupal_strstr($key, 'shipping_') !== FALSE) {
            $name = drupal_substr($key, drupal_strlen('shipping_'));
            if (empty($edit["billing_$name"])) {
              $edit["billing_$name"] = $value;
              $reset_bc = TRUE;
            }
          }
        }
      }
      unset($billing_error);
    }

    if (!empty($reset_bc)) {
      $edit['billing_country'] = $edit['shipping_country'];
    }

    switch ($validate) {
      case 'shipping':
        if (!empty($shipping_error)) {
          $errors['shipping'] = $shipping_error;
        }
        break;

      case 'billing':
        if (!empty($billing_error)) {
          $errors['billing'] = $billing_error;
        }
        break;

      case 'all':
        if (!empty($shipping_error)) {
          $errors['shipping'] = $shipping_error;
        }

        if (!empty($billing_error)) {
          $errors['billing'] = $billing_error;
        }
        break;
    }
  }

  foreach ($errors as $name => $message) {
    form_set_error($name, $message);
  }
  return count($errors) == 0;
}

function ec_store_transaction_validate_item(&$edit) {

  $errors = array();

  if ($edit['nids'] != '' && !preg_match("/^[0-9,\s]+$/", $edit['nids'])) {
    $errors['nids'] = t('Please enter only numeric values or commas in the product id field.');
  }

  if (!empty($edit['item'])) {
    foreach ($edit['item'] as $key => $value) {

      $edit['item'][$key]['nid'] = $key;
      foreach (explode("\n", $edit['item'][$key]['data']) as $d) {

        $d = explode(":", $d);
        if (!empty($d[0])) {
          $arr[$d[0]] = trim($d[1]);
        }
      }
      $edit['item'][$key]['data'] = $arr;
      $edit['item'][$key] = (object)($edit['item'][$key]);

      if (strtotime($edit['item'][$key]->expires) != -1) {
        $edit['item'][$key]->expires = strtotime($edit['item'][$key]->expires);
      }
    }
  }

  foreach ($errors as $name => $message) {
    form_set_error($name, $message);
  }
  return count($errors) == 0;
}

function ec_store_transaction_overview_form($edit) {

  if (is_numeric($edit->uid)) {
    $account = user_load(array('uid' => $edit->uid));
    $edit->uid = $edit->uid == 0 ? variable_get('anonymous', 'Anonymous') : $account->name;
  }

  if (empty($edit->mail)) {
    $edit->mail = $account->mail;
  }

  if (!empty($edit->changed)) {
    $edit->changed = format_date($edit->changed);
  }

  $form['overview'] = array(
    '#type' => 'fieldset',
    '#title' => t('Invoice overview'),
    );
  $form['overview']['workflow'] = array(
    '#type' => 'select',
    '#title' => t('Transaction workflow'),
    '#default_value' => $edit->workflow,
    '#options' => ec_store_transaction_workflow(),
  );
  $form['overview']['uid'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => $edit->uid,
    '#size' => 30,
    '#maxlength' => 60,
    '#description' => t('Enter either an username or e-mail address below.'),
  );
  $form['overview']['mail'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail address'),
    '#default_value' => $edit->mail,
    '#size' => 30,
    '#maxlength' => 64,
  );
  $form['overview']['gross'] = array(
    '#type' => 'item',
    '#title' => t('Total cost'),
    '#value' => format_currency($edit->gross),
  );
  if (!empty($edit)) {
    $form['overview']['changed'] = array(
      '#type' => 'item',
      '#title' => t('Last modified'),
      '#value' => $edit->changed,
      '#size' => 30,
      '#maxlength' => 30,
    );
  }

  return $form;
}

/**
 * Creates a standard address form
 * When passed a display field we filter out all other sections and display only
 * the one given. If passed shipping only a shipping address will be shown.
 *
 * @param $txn
 *   Objectm this is a transaction object used for default values
 * @param $display
 *   String, option field that allows the form to filter sections
 */
function ec_store_transaction_addresses_form($txn, $display = NULL) {
  $form['address']['#tree'] = TRUE;

  $form['address']['shipping'] = ec_store_address_form($txn->address['shipping']);
  $form['address']['shipping']['#type'] = 'fieldset';
  $form['address']['shipping']['#title'] = t('Shipping address');

  $form['address']['billing'] = ec_store_address_form($txn->address['billing']);
  $form['address']['billing']['#type'] = 'fieldset';
  $form['address']['billing']['#title'] = t('Billing address');

  switch ($display) {
    case 'shipping':
      unset($form['billing']);
      break;

    case 'billing':
      unset($form['shipping']);
      break;
  }
  return $form;
}

function ec_store_address_form($address) {
  module_load_include('inc', 'ec_store', 'ec_store.localization');

  if (isset($address->aid)) {
    $form['type'] = array(
      '#type'  => 'hidden',
      '#value' => $address->type,
    );
  }

  $form['firstname'] = array(
    '#type'          => 'textfield',
    '#title'         => t('First name'),
    '#default_value' => $address->firstname,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['lastname'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Last name'),
    '#default_value' => $address->lastname,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['country'] = array(
    '#type'          => 'select',
    '#title'         => t('Country'),
    '#default_value' => ($address->country ? $address->country : variable_get('ec_country', '')),
    '#options'       => _ec_store_location_countries(),
  );
  $form['street1'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Street'),
    '#default_value' => $address->street1,
    '#size'          => 60,
    '#maxlength'     => 64,
  );
  $form['street2'] = array(
    '#type'          => 'textfield',
    '#default_value' => $address->street2,
    '#size'          => 60,
    '#maxlength'     => 64,
  );
  $form['city'] = array(
    '#type'          => 'textfield',
    '#title'         => t('City'),
    '#default_value' => $address->city,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['state'] = array(
    '#type'          => 'textfield',
    '#title'         => t('State/Province'),
    '#default_value' => $address->state,
    '#size'          => 32,
    '#maxlength'     => 32,
  );
  $form['zip'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Zip/Postal Code'),
    '#default_value' => $address->zip,
    '#size'          => 10,
    '#maxlength'     => 10,
  );
  return $form;
}

function ec_store_transaction_item_form($edit) {
  $form = NULL;
  if (is_array($edit->item)) {
    foreach ($edit->item as $k => $p) {

      $form[$k] = array(
        '#type' => 'fieldset',
        '#title' => t('Item #%nid', array('%nid' => $p->nid))
        );
      $form[$k]['item'][$p->nid]['title'] = array(
        '#type' => 'textfield',
        '#title' => t('Title'),
        '#default_value' => $p->title,
        '#size' => 30,
        '#maxlength' => 128,
        );
      $form[$k]['item'][$p->nid]['qty'] = array(
        '#type' => 'textfield',
        '#title' => t('Quantity'),
        '#default_value' => $p->qty,
        '#size' => 15,
        '#maxlength' => 15,
        );
      $form[$k]['item'][$p->nid]['qty'] = array(
        '#type' => 'textfield',
        '#title' => t('Quantity'),
        '#default_value' => $p->qty,
        '#size' => 15,
        '#maxlength' => 15,
        );
      $form[$k]['item'][$p->nid]['data'] = array(
        '#type' => 'hidden',
        '#value' => $p->data,
        );

      $interval = drupal_map_assoc(range(0, 31));
      $interval[0] = '--';
      $unit = array(0 => t('-please choose-'), 'D' => t('Day(s)'), 'W' => t('Week(s)'), 'M' => t('Month(s)'), 'Y' => t('Year(s)'));

      if (!empty($p->expires)) {
        $p->expires = format_date($p->expires, 'custom', 'Y-m-d H:i O');
      }

      if (!empty($p->is_recurring)) {
        $form[$k]['recurring'] = array(
          '#type' => 'fieldset',
          '#title' => t('Billing cycle length'),
          );
        $form[$k]['recurring']['item'][$p->nid]['price_interval'] = array(
          '#type' => 'select',
          '#title' => t('Interval'),
          '#default_value' => $p->price_interval,
          '#options' => $interval,
          );
        $form[$k]['recurring']['item'][$p->nid]['price_unit'] = array(
          '#type' => 'select',
          '#title' => t('Unit'),
          '#default_value' => $p->price_unit,
          '#options' => $unit,
          );
        $interval[0] = 0;
        $interval[-1] = t('unlimited');
        $form[$k]['recurring']['item'][$p->nid]['price_cycle'] = array(
          '#type' => 'select',
          '#title' => t('Number of payments cycles'),
          '#default_value' => $p->price_cycle,
          '#options' => $interval,
          '#description' => t('Select the number of recurring payment cycles this product has.'),
          );
        $form[$k]['recurring']['item'][$p->nid]['expires'] = array(
          '#type' => 'textfield',
          '#title' => t('expires'),
          '#default_value' => $p->expires,
          '#size' => 21,
          '#maxlength' => 25,
          '#description' => t('The date the product will expire, or if using recurring payments, the date of the next billing cycle unless all payment cycles have ended.'),
          );
      }
    }
  }
  else {
    $form['products'] = array(
      '#type' => 'fieldset',
      '#title' => t('Products'),
      );
    $form['products']['nids'] = array(
      '#type' => 'textfield',
      '#title' => t('Product IDs'),
      '#default_value' => $edit->nids,
      '#autocomplete_path' => 'product/autocomplete/all',
      '#size' => 60,
      '#maxlength' => 128,
      '#description' =>  t('Enter a comma separated list of product ids to add to this transaction. Here is a <a href="!product_quicklist" onclick="window.open(this.href, \'!product_quicklist\', \'width=480,height=480,scrollbars=yes,status=yes\'); return false">list of all products</a>.', array('!product_quicklist' => url('admin/store/products/quicklist')))
      );
  }

  return $form;
}

/**
 * Format a shipping method.
 *
 * @param $txn
 *   Object containing shipping information.
 */
function ec_store_format_shipping_method($txn) {
  $methods = module_invoke('shipping', 'get_methods');
  if (!empty($txn->ship) && is_array($methods)) {
    $method = $methods[$txn->ship->service][$txn->ship->method];
    return t('!service !method', array('!service' => $methods[$txn->ship->service]['#title'],
      '!method' => $methods[$txn->ship->service][$txn->ship->method]['#title']));
  }
  return t('none');
}
