<?php
// $Id: ec_store.manage.inc,v 1.2 2008/11/01 04:32:09 gordon Exp $

/**
 * @file
 * store.manage.inc
 *   This file provides store management displays and features. It only loads
 *   where the current user is a manager, and on store management pages.
 *   (In contrast, store.admin.inc only loads for store configuration pages.)
 */

// -------------------------------------------------------------------------
// Transaction header form

/**
 * Valid sections are
 *    - overview: changing workflow, timestamps, and totals
 *      admin/store/transaction
 *
 *    - addresses: shipping and billing info
 *      admin/store/transaction/address
 *
 *    - items: editing and adding items
 *      admin/store/transaction/item/
 */
function ec_store_transaction_form($txnid) {
  if ($txn = ec_store_transaction_load($txnid)) {
    module_load_include('inc', 'ec_store', 'ec_store.admin');
    return _ec_store_transaction_form($txn);
  }
  else {
    drupal_not_found();
    exit();
  }
}

/**
 * Called by ec_store_transaction_form()
 */
function _ec_store_transaction_form($edit = NULL) {
  module_load_include('inc', 'ec_store', 'ec_store');
  $edit = is_array($edit) ? (object) $edit : $edit;

  $section = $edit->section ? $edit->section : $_GET['section'];

  switch (drupal_strtoupper($section)) {
    case 'OVERVIEW':
      $form = ec_store_transaction_overview_form($edit);
      break;

    case 'ADDRESSES':
      $form = ec_store_transaction_addresses_form($edit);
      break;

    case 'ITEMS':
      $form = ec_store_transaction_item_form($edit);
      break;

    default:
      $form = array_merge(ec_store_transaction_overview_form($edit),
        ec_store_transaction_addresses_form($edit), ec_store_transaction_item_form($edit));
  }

  if ($edit->uid && is_numeric($edit->uid)) {
    $edit->uid = db_result(db_query('SELECT name FROM {users} WHERE uid = %d', $edit->uid));
  }

  if (!empty($edit->txnid)) {
    $form['txnid'] = array(
      '#type' => 'value',
      '#value' => $edit->txnid,
      );
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Update transaction')
      );
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      );
  }
  else {
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Create new transaction'),
      );
  }

  $form['section'] = array(
    '#type' => 'value',
    '#value' => $section,
    );

  return $form;
}

function ec_store_transaction_form_validate(&$form, &$form_state) {
  ec_store_transaction_validate($form_state['values']);
}

function ec_store_transaction_validate(&$edit) {
  $edit['section'] = $edit['section'] ? drupal_strtoupper($edit['section']) : 'ALL';

  switch ($edit['section']) {

    case 'OVERVIEW':
      ec_store_transaction_validate_overview($edit);
      break;

    case 'ADDRESSES':
      ec_store_transaction_validate_addresses($edit);
      break;

    case 'ITEMS':
      ec_store_transaction_validate_item($edit);
      break;

    default:
      ec_store_transaction_validate_overview($edit);
      ec_store_transaction_validate_addresses($edit);
      ec_store_transaction_validate_item($edit);
  }

  ec_store_invoke_ec_transactionapi($edit, 'validate', drupal_strtoupper($edit['section']));

  return ((form_get_errors()) ? FALSE : TRUE);
}

function ec_store_transaction_form_submit(&$form, &$form_state) {
  if ($form_state['values']['op'] == t('Delete')) {
    $dest = drupal_get_destination();
    unset($_REQUEST['destination'], $_REQUEST['edit']['destination']);
    $form_state['redirect'] = array('admin/store/transaction/delete/'. $form_state['values']['txnid'], $dest);
  }
  else {
    ec_store_transaction_save((object)$form_state['values']);
    form_state['redirect'] = 'admin/store/transaction';
  }
}

// -------------------------------------------------------------------------
// Transaction items form

function ec_store_transaction_edit_items($txnid, $nid = NULL) {
  if (!is_numeric($txnid) || !($t = ec_store_transaction_load($txnid))) {
    return drupal_not_found();
  }

  if (is_null($nid) || !is_numeric($nid)) {
    return drupal_get_form('ec_store_transaction_add_items_form', $t);
  }
  else {
    $p = NULL;
    foreach (array_keys($t->items) as $k) {
      $p =& $t->items[$k];
      if ($p->nid == $nid) {
        break;
      }
    }

    if (is_null($p)) {
      return drupal_not_found();
    }

    return drupal_get_form('ec_store_transaction_edit_items_form', $t, $p);
  }
}

/**
 * Displays the form for editing a product already attached to a transaction.
 *
 * @todo: add hooks to this form for recurring etc
 *
 * @param $t Transaction
 * @param $p Product
 * @return HTML form for editing the given product as part of the given transaction.
 */
function ec_store_transaction_edit_items_form(&$t, &$p) {
  $form['txnid'] = array(
    '#type' => 'value',
    '#value' => $t->txnid
  );
  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $p->nid
  );
  $form['vid'] = array(
    '#type' => 'value',
    '#value' => $p->vid
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $p->title,
    '#size' => 30,
    '#maxlength' => 128,
  );
  $form['qty'] = array(
    '#type' => 'textfield',
    '#title' => t('Quantity'),
    '#default_value' => $p->qty,
    '#size' => 15,
    '#maxlength' => 15,
  );
  $form['data'] = array(
    '#type' => 'hidden',
    '#value' => $p->data,
  );

  $interval = drupal_map_assoc(range(0, 31));
  $interval[0] = '--';
  $unit = array(0 => t('-please choose-'),
    'D' => t('Day(s)'), 'W' => t('Week(s)'),
    'M' => t('Month(s)'), 'Y' => t('Year(s)'));

  $form[] = array('#type' => 'submit', '#value' => t('Update'), '#weight' => 2);

  return $form;
}

function ec_store_transaction_edit_items_form_submit(&$form, &$form_state) {
  product_transaction_save_product((object)$form_state['values']);

  // Deleting the cache
  ec_store_transaction_cache($form_state['values']['txnid'], NULL, TRUE);

  // Fix up Gross
  $txn = ec_store_transaction_load($form_state['values']['txnid']);
  $txn->gross = ec_store_transaction_calc_gross($txn);
  db_query('UPDATE {ec_transaction} SET gross = %f WHERE txnid = %d', $t->gross, $t->txnid);
}


function ec_store_transaction_items($txnid) {
  $html = ec_store_quicklist_header();

  $header = array(
    array('data' => t('nid'), 'field' => 'p.nid'),
    array('data' => t('sku'), 'field' => 'p.sku'),
    array('data' => t('title'), 'field' => 'stp.title', 'sort' => 'asc'),
    array('data' => t('qty'), 'field' => 'stp.qty'),
    array('data' => t('type'), 'field' => 'p.ptype')
  );

  if ($count = db_result(db_query('SELECT COUNT(p.nid)
      FROM {ec_transaction_product} as stp, {ec_product} as p
      WHERE stp.vid = p.vid AND stp.txnid = %d', $txnid))) {
    $sql = 'SELECT * FROM {ec_transaction_product} as stp, {ec_product} as p
      WHERE stp.vid = p.vid AND stp.txnid = %d'. tablesort_sql($header);
  }
  else {
    $sql = 'SELECT * FROM {ec_transaction_product} as stp WHERE stp.txnid = %d'. tablesort_sql($header);
  }
  $result = pager_query($sql, 50, 0, NULL, $txnid);

  $num_items = 0;
  while ($data = db_fetch_object($result)) {
    ++$num_items;
    $qty += $data->qty;

    $rows[] = array($data->nid, check_plain($data->sku), l($data->title, "node/$data->nid", array('target' => '_blank')), $data->qty, $data->ptype);
  }

  $r[] = array(array('data' => 'Unique items:', 'align' => 'right'),
    array('data' => "<strong>$num_items</strong>"));
  $r[] = array(array('data' => 'Total quantity:', 'align' => 'right'),
    array('data' => "<strong>$qty</strong>"));
  $table = theme('table', NULL, $r, array('border' => 0));

  $rows[] = array(array('data' => $table, 'colspan' => 5, 'align' => 'right'));

  $pager = theme('pager', NULL, 50, 0);
  if (!empty($pager)) {
    $rows[] = array(array('data' => $pager, 'colspan' => 5));
  }

  echo $html;
  if (!empty($rows)) {
    echo theme('table', $header, $rows);
  }
  else {
    echo '<p>'. t('There are no transactions to view. You can manually <a href="!add_transaction">add transactions</a>, however.', array('!add_transaction' => 'admin/store/transaction/add')) .'</p>';
  }
  echo '</body></html>';
}

// -------------------------------------------------------------------------
// Transaction delete form

/**
 * Confirm users wants to delete transaction using confirm_form().
 * @param $txnid
 *   Transaction ID of transaction to be deleted.
 */
function ec_store_transaction_confirm_delete($txnid) {
  $form['txnid'] = array(
    '#type' => 'value',
    '#value' => $txnid,
  );
  return confirm_form(
    $form,
    t('Are you sure you want to delete this transaction?'),
    'admin/store/transaction',
    t('Are you sure you want to delete transaction #%txnid?  This cannot be undone.',
    array('%txnid' => $txnid)),
    t('Confirm delete'),
    t('Cancel'));
}

/**
 * hook_submit for ec_store_transaction_confirm_delete().  Performs delete of
 * transaction using ec_store_transaction_delete().
 */
function ec_store_transaction_confirm_delete_submit(&$form, &$form_state) {
  ec_store_transaction_delete($form_state['values']['txnid']);
  $form_state['redirect'] = 'admin/store/transaction';
}

/**
 * Delete a transaction from the database and trigger ec_transactionapi.
 *
 * @param $txnid
 *  Transaction id of the transaction to delete.
 */
function ec_store_transaction_delete($txnid) {
  $txn = ec_store_transaction_load($txnid);
  ec_store_invoke_ec_transactionapi($txn, 'delete');
  db_query('DELETE FROM {ec_transaction} WHERE txnid = %d', $txnid);

  // Purge the transactions cache so that the the next load won't find this
  // transactions that hase been loaded.
  ec_store_transaction_cache($txnid, NULL, TRUE);
}

function ec_store_transaction_add_items_form(&$txn) {
  $items = ec_store_transaction_item_list($txn);

  if (empty($items)) {
    $items = t('No items found for the transaction');
  }
  $form['items'] = array(
    '#type' => 'item',
    '#title' => t('Transaction items'),
    '#value' => $items
  );
  $form['txnid'] = array(
    '#type' => 'value',
    '#value' => $txn->txnid
  );

  $form['products'] = array(
    '#type' => 'fieldset',
    '#title' => t('Products'),
  );
  $form['products']['nids'] = array(
    '#type' => 'textfield',
    '#title' => t('Product IDs'),
    '#default_value' => $txn->nids,
    '#autocomplete_path' => 'product/autocomplete/all',
    '#size' => 60,
    '#maxlength' => 128,
    '#description' =>  t('Enter a comma separated list of product ids to add to this transaction. Here is a <a href="!product_quicklist" onclick="window.open(this.href, \'!product_quicklist\', \'width=480,height=480,scrollbars=yes,status=yes\'); return FALSE">list of all products</a>.', array('!product_quicklist' => url('admin/store/products/quicklist')))
  );
  $form['products'][] = array('#type' => 'submit', '#value' => t('Add products'));

  return $form;
}

function ec_store_transaction_item_list(&$txn) {
  $rows = array();
  if (!empty($txn->items)) {
    foreach (array_keys($txn->items) as $key) {
      $p =& $txn->items[$key];
      $rows[] = array($p->title, l(t('edit'), "admin/store/transaction/product/edit/$txn->txnid/$p->nid",
        array(), drupal_get_destination()));
    }
  }

  if (!empty($rows)) {
    return theme('table', array('product', 'op'), $rows);
  }

  return '';
}

function ec_store_transaction_add_items_form_validate(&$form, &$form_state) {
  foreach (split(', ?', $form_state['values']['nids']) as $nid) {
    if (!is_numeric($nid) || !db_result(db_query('SELECT nid FROM {ec_product}
        WHERE nid = %d', $nid))) {
      form_set_error('nids', t('Invalid list of product IDs'));
      break;
    }
  }
}

function ec_store_transaction_add_items_form_submit(&$form, &$form_state) {
  $form_state['values']['nids'] = drupal_str_replace(' ', '', $form_state['values']['nids']);
  ec_store_transaction_save_nids($form_values);
}

// -------------------------------------------------------------------------
// e-Commerce search hooks (see ec_common.manage.inc)

/**
 * Implementation of hook_ec_search_init().
 * (called from ec_common.manage.inc)
 */
function ec_store_transaction_ec_search_init(&$query) {
  $query->ensure_table('ec_transaction', 'st');
  $query->add_select('st.txnid');
  $query->add_select('st.*');
}

/**
 * Implementation of hook_ec_search().
 * (Called from ec_common.manage.inc)
 */
function ec_store_ec_search($search_id, &$query, &$filter, &$details) {
  switch ($search_id) {
    case 'ec_store_transaction':
      switch ($filter['name']) {
        case 'text':
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('node', 'n', 'LEFT', 'stp.vid = n.vid');
          $query->ensure_table('node_revisions', 'r', 'LEFT', 'stp.vid = r.vid');
          $query->add_where("(n.title LIKE '%%{$filter['data'][0]}%%' OR r.body LIKE '%%{$filter['data'][0]}%%')");
          $details[] = ec_search_detail(t('Text'), $filter);
          break;

        case 'txnid':
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'st.txnid'));
          $details[] = ec_search_detail(t('Transaction id'), $filter);
          break;

        case 'nid':
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('ec_product', 'p', 'INNER', 'stp.vid = p.vid');
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'p.nid'));
          $details[] = ec_search_detail(t('Product id'), $filter);
          break;

        case 'nid_expires':
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('ec_product', 'p', 'INNER', 'stp.vid = p.vid');
          $mod = array();
          foreach ($filter['data'] as $key => $value) {
            if ($key !== 'op') {
              $mod[$key] = format_date(strtotime($value), 'small');
            }
          }
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'stp.expires', 'strtotime'));
          $details[] = ec_search_detail(t('Product expires'), $filter, $mod);
          break;

        case 'date':
          $mod = array();
          foreach ($filter['data'] as $key => $value) {
            if ($key !== 'op') {
              $mod[$key] = format_date(strtotime($value), 'small');
            }
          }
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'st.created', 'strtotime'));
          $details[] = ec_search_detail(t('Date'), $filter, $mod);
          break;

        case 'duedate':
          $mod = array();
          foreach ($filter['data'] as $key => $value) {
            if ($key !== 'op') {
              $mod[$key] = format_date(strtotime($value), 'small');
            }
          }
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'st.duedate', 'strtotime'));
          $query->add_where('AND st.duedate != 0');
          $details[] = ec_search_detail(t('Date Due'), $filter, $mod);
          break;

        case 'gross':
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'st.gross'));
          $details[] = ec_search_detail(t('Cost'), $filter);
          break;

        case 'allocation':
          $mod = array();
          foreach ($filter['data'] as $key => $value) {
            $value = ($key !== 'op') ? transaction_get_allocation($value) : $value;
            $mod[$key] = $value;
          }
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'st.allocation'));
          $details[] = ec_search_detail(t('Allocation'), $filter, $mod);
          break;

        case 'workflow':
          $mod = array();
          foreach ($filter['data'] as $key => $value) {
            $value = ($key !== 'op') ? transaction_get_workflow($value) : $value;
            $mod[$key] = $value;
          }
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'st.workflow'));
          $details[] = ec_search_detail(t('Workflow'), $filter, $mod);
          break;

        case 'ptype':
          $mod = array_intersect_key(product_ptypes_get('names'), array_flip($filter['data']));
          sort($mod);
          $query->ensure_table('ec_transaction_product', 'stp', 'INNER', 'st.txnid = stp.txnid');
          $query->ensure_table('ec_product', 'p', 'INNER', 'stp.vid = p.vid');
          $query->add_where(ec_search_operator($filter['data'], $filter['op'], 'p.ptype'));
          $details[] = ec_search_detail(t('Product type'), $filter, $mod);
          break;
      }
      break;
  }
}

/**
 * Implementation of hook_ec_search_format_header().
 * (Called from ec_common.manage.inc)
 */
function ec_store_ec_search_format_header($search_id, $criteria, $query, &$header) {
  if ($search_id == 'ec_store_transaction') {
    $header['txnid'] = array(
      'data'    => t('ID'),
      'field'   => 'st.txnid',
      'sort'    => 'desc',
      'weight'  => -10,
    );
    $header['workflow'] = array(
      'data'    => t('Workflow'),
      'field'   => 'st.workflow',
      'weight'  => 2,
    );
    $header['gross'] = array(
      'data'    => t('Total'),
      'field'   => 'st.gross',
      'weight'  => 3,
    );
    $header['created'] = array(
      'data'    => t('Created'),
      'field'   => 'st.created',
      'weight'  => 5,
    );
    $header['changed'] = array(
      'data'    => t('Changed'),
      'field'   => 'st.changed',
      'weight'  => 6,
    );
    $header['datedue'] = array(
      'data'    => t('Due date'),
      'field'   => 'st.duedate',
      'weight'  => 7,
    );
    $header['items'] = array(
      'data'    => t('Items ordered'),
      'weight'  => 98,
    );
  }
  $header['op'] = array(
    'data'    => t('Operations'),
    'weight'  => 99,
  );
}

/**
 * Implementation of hook_ec_search_format_row().
 * (Called from ec_common.manage.inc)
 */
function ec_store_ec_search_format_row($search_id, $criteria, $query, $header, &$row, $data) {
  if (isset($data->txnid)) {
    $row[0]['txnid'] = array(
      'data'  => $data['txnid'],
      'align' => 'right',
    );
  }
  if (isset($header['ecid'])) {
    if (!isset($row[0]['ecid'])) {
      $row[0]['ecid'] = array('data' => array());
    }
    if ($search_id == 'ec_store_transaction') {
      if ($billing_address = db_fetch_object(db_query('SELECT *
        FROM {ec_transaction_address} WHERE txnid = %d
        AND type = "billing"', $data['txnid']))) {
        if ($billing_address->firstname || $billing_address->lastname) {
          $name = $billing_address->firstname .' '. $billing_address->lastname;
          $name = l($name, 'admin/store/transaction/edit/'. $data['txnid'],
            array(), 'section=addresses&'. drupal_get_destination());
        }
        else {
          $name = $billing_address->fullname;
          $name = l($name, 'admin/store/transaction/edit/'. $data['txnid'],
            array(), 'section=addresses&'. drupal_get_destination());
        }
      }
      $row[0]['ecid']['data']['address'] = array(
        '#value' => $name,
      );
      $row[0]['ecid']['nowrap'] = 'nowrap';
    }
    $actions = array(
      'print_invoice' => l(t('print invoice'), "admin/store/transaction/print/{$data['txnid']}", array(), ($data->$col > 1 ? NULL : 'wf=2')),
      'ship_notify' => l(t('shipping notify'), "admin/store/transaction/shipping/{$data['txnid']}", array(), 'wf=3&'. drupal_get_destination()),
      'ask_customer' => l(t('ask customer'), "admin/store/transaction/ask/{$data['txnid']}", array(), 'wf=4&'. drupal_get_destination()),
      'cancel_transaction' => l(t('cancel transaction'), "admin/store/transaction/cancel/{$data['txnid']}", array(), 'wf=5&'. drupal_get_destination())
    );
    $row[0]['workflow']['data']['wf'] = array(
      '#type' => 'fieldset',
      '#title' => transaction_get_workflow($data['workflow']),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      'actions' => array(
        '#value' => theme('item_list', $actions),
      ),
    );
    $row[0]['gross']['data'] = array(
      '#value' => format_currency($data['gross']),
    );
    $row[0]['gross']['align'] = 'right';
    $row[0]['created']['data'] = array(
      '#value' => format_date($data['created'], 'small'),
    );
    $row[0]['changed']['data'] = array(
      '#value' => format_date($data['changed'], 'small'),
    );
    $row[0]['datedue']['data'] = array(
      '#value' => $data['duedate'] ? format_date($data['duedate'], 'small') : '',
    );
    $items = db_result(db_query('SELECT SUM(qty) FROM {ec_transaction_product}
      WHERE txnid = %d', $data['txnid']));
    $row[0]['items']['data'] = array(
      '#value' => l(format_plural($items, '1 item', '@count items'),
      'admin/store/transaction/product/edit/'. $data['txnid'],
      array('onclick' => 'window.open(this.href, \'items_ordered\',
      \'width=480,height=480,scrollbars=yes,status=yes\'); return FALSE')),
    );
    $links = module_invoke_all('link', $search_id, (object)$data);
    $row[0]['op']['data'] = array(
      '#value' => theme('links', $links),
    );
  }
}

// -------------------------------------------------------------------------
// eC Mail

/**
 * Implementation of hook_mail_reset() from ec_mail module.
 */
function ec_store_mail_reset($type) {
  $mids = array();
  $mid = NULL;
  $var = NULL;

  switch ($type) {
    case ECMAIL_TYPE_CUSTOMER_INVOICE:
      $defsub = t('Your %site order');
      $defbody = t("Dear %billing_name,\n\nThanks for your recent purchase from %site.  This message includes important information about your order. Please take a moment to read it closely, and be sure to save a copy for future reference.\n\n********************************\nBilling and Shipping Information\n********************************\n%user_data\n********************************\nOrder Details\n********************************\n%txn_items\nQuestions about your order? Please contact us at %email\n\n********************************\nShipping Instructions\n********************************\nWe will notify you by email as soon as your order ships.\n\nThanks for shopping at %site.  We hope to hear from you again real soon!\n\n%uri");
      $mid = ec_mail_import_old_mail(t('Default customer invoice'), $type, 'ec_store_email_customer_invoice_subject', $defsub, 'ec_store_email_customer_invoice_body', $defbody);
      $var = MAILVAR_CUSTOMER_INVOICE;
      break;
    case ECMAIL_TYPE_PROCESSING_ERROR:
      $defsub = t('Purchase problem, %site');
      $defbody = t("Dear Customer\n\nIt seems there was a problem while processing your order (%txnid). Please contact us at %email for further details.\n\nRegards,\n%site team\n%uri");
      $mid = ec_mail_import_old_mail(t('Default processing error'), $type,
        'ec_store_email_processing_error_subject', $defsub,
        'ec_store_email_processing_error_body', $defbody);
      $var = MAILVAR_PROCESSING_ERROR;
      break;
    case ECMAIL_TYPE_CANCEL_TXN:
      $defsub = t('Your %site order has been canceled');
      $defbody = t("Dear %billing_name,\n\nYour order (%txnid) has been canceled. Please contact us at %email for further details.\n\nRegards,\n%site team\n%uri");
      $mid = ec_mail_import_old_mail(t('Default order cancellation notice'),
        $type, 'ec_store_email_cancel_transaction_subject', $defsub,
        'ec_store_email_cancel_transaction_body', $defbody);
      $var = MAILVAR_CANCEL_TXN;
      break;
    case ECMAIL_TYPE_ASK_CUSTOMER:
      $defsub = t('Questions regarding your order from %site');
      $defbody = t("Dear %billing_name,\n\nWe have some questions regarding your order from %site.\n\nRegards,\n%site team\n%uri");
      $mid = ec_mail_import_old_mail(t('Default query to customer'), $type,
        'ec_store_email_ask_customer_template_subject', $defsub,
        'ec_store_email_ask_customer_template_body', $defbody);
      $var = MAILVAR_ASK_CUSTOMER;
      break;
  }

  if (!empty($var)) {
    ec_mail_variable_change($var, $mid, TRUE);
  }

  if (!empty($mid)) {
    $mids[] = $mid;
  }

  return $mids;
}

/**
 * Implementation of hook_mail_types() from ec_mail module.
 */
function ec_store_mail_types() {
  return array(
    ECMAIL_TYPE_CUSTOMER_INVOICE => t('Customer invoice'),
    ECMAIL_TYPE_PROCESSING_ERROR => t('Processing error notification'),
    ECMAIL_TYPE_CANCEL_TXN => t('Transaction cancelled notice'),
    ECMAIL_TYPE_ASK_CUSTOMER => t('Query to customer'),
  );
}

/**
 * Send an invoice email.
 * @param $txn
 *   Transaction to send email based on.
 * @param $preview
 *   Boolean, this toggles whether the user wants to see a preview.
 */
function ec_store_send_invoice_email($txn, $preview = NULL) {
  if ($txn->mail && valid_email_address($txn->mail)) {
    $to = $txn->mail;
  }
  else {
    $to = ec_customer_get_email($txn->ecid);
  }

  $mid = variable_get(MAILVAR_CUSTOMER_INVOICE, 0);
  if (!empty($preview)) {
    return ec_mail_preview($mid, $to, $txn);
  }
  elseif (!empty($to) && valid_email_address($to)) {
    return ec_mail_send_mid($mid, $to, $txn);
  }
  else {
    return FALSE;
  }
}

function ec_store_send_error_email($txnid, $preview = NULL) {
  $txn = ec_store_transaction_load($txnid);

  if (!empty($txn->mail) && valid_email_address($txn->mail)) {
    $to = $txn->mail;
  }
  else {
    $to = ec_customer_get_email($txn->ecid);
  }

  $mid = variable_get(MAILVAR_PROCESSING_ERROR, 0);

  if (!empty($preview)) {
    return ec_mail_preview($mid, $to, $txn);
  }
  elseif (!empty($to) && valid_email_address($to)) {
    return ec_mail_send_mid($mid, $to, $txn);
  }
  else {
    return FALSE;
  }
}
/**
 * Ask customer question by mail workflow page.
 * @todo: this could be moved completely into a menu entry.
 */
function ec_store_transaction_ask_customer() {
  return drupal_get_form('ec_store_transaction_mail_form', arg(4),
    t('Ask customer a question'), MAILVAR_ASK_CUSTOMER);
}


/**
 * FIXME: Preview is not working.
 * Displays a mail form for the given mail variable and allows the user to preview
 * the mail with tokens substituted. Only use this function for mails that
 * populate tokens through a transaction object.
 *
 * @param $txnid
 *  Transaction ID.
 * @param $pagetitle
 *  Title to set for the page using drupal_set_title().
 * @param $variable
 *  Variable name for the value of the mail ID.
 * @param $force_wf
 *  Forces the workflow to be changed if the mail is sent. Only works if $_GET['wf']
 *  contains a workflow value. Default is FALSE.
 * @param $send_optional
 *  TRUE if the mail sending is optional. Default is FALSE.
 */
function ec_store_transaction_mail_form($txnid, $pagetitle, $variable, $force_wf = FALSE, $send_optional = FALSE) {
  $txn = ec_store_transaction_load($txnid);

  if (!empty($txn->mail) && valid_email_address($txn->mail)) {
    $to = $txn->mail;
  }
  else {
    $to = ec_customer_get_email($txn->customer);
  }

  $form = ec_mail_send_varmail_form($variable, $to, $pagetitle, $txn);

  $form['txnid'] = array(
    '#type' => 'value',
    '#value' => $txn->txnid,
  );

  if (!empty($send_optional)) {
    unset($form['perform_send']);
    $form['perform_send'] = array(
      '#type' => 'checkbox',
      '#title' => t('Notify customer'),
      '#description' => t('Check here if you want a mail notification sent to the customer.'),
      '#default_value' => FALSE,
    );
  }

  if (($workflow = $_GET['wf']) && transaction_valid_workflow($workflow)) {
    if (empty($force_wf)) {
      $form['change_workflow'] = array(
        '#type' => 'checkbox',
        '#title' => t('Change Workflow'),
        '#description' => t('Check here if you want to change the transaction workflow to <strong>%new_workflow</strong>.', array('%new_workflow' => transaction_get_workflow($workflow))),
        '#default_value' => TRUE,
      );
    }
    else {
      // we're forcing the workflow, so we still need to set the change_workflow value.
      $form['change_workflow'] = array('#type' => 'value', '#value' => TRUE);
    }
    $form['wf'] = array('#type' => 'value', '#value' => $workflow);
  }

  return $form;
}

function ec_store_transaction_mail_form_submit(&$form, &$form_state) {
  if (!empty($form_state['values']['perform_send'])) {
    drupal_set_message(t('The mail has been accepted for delivery to the customer.'));
  }

  if (!empty($form_state['values']['change_workflow']) && is_numeric($form_state['values']['wf'])) {
    $txn = new StdClass;
    $txn->txnid = $form_state['values']['txnid'];
    $txn->workflow = $form_state['values']['wf'];
    ec_store_transaction_save($txn);
    drupal_set_message(t('The transaction workflow has been updated.'));
  }
}
